library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

entity Counter is
port (
      clk                   : in std_logic;
      reset                 : in std_logic;
      direction             : in std_logic:='1';
      digit_selection_in    : in std_logic:='1';                     -- Digit Selection IN Pin for PmodSSD
      led_result            : out std_logic_vector(3 downto 0); -- LED Result input
      seven_segment         : out std_logic_vector(6 downto 0); -- 7-segment output MSBit is g, LSBit is a
      digit_selection_out   : out std_logic                     -- Digit Selection OUT Pin for PmodSSD
    );
end entity Counter;

architecture Behavioral of Counter is

signal clk_one_sec: std_logic;                             --NEW CLOCK. Ticks every sec  --OK
signal new_clk_counts:  integer:=0;
signal clk_counts:     integer:=0; 
signal sec_counts:      unsigned(3 downto 0):=(others=>'0');                      --Running timer value
--signal sec_counts:      integer:=0;                      --Running timer value
signal new_sec_counts: unsigned(3 downto 0):=(others=>'0'); 
--signal new_sec_counts: integer:=0;
signal next_counts: unsigned(3 downto 0);                      --Running timer value

begin

--FIRST LOGIC CIRCUIT
--COUNTS UPTO 100M clock ticks

--1st Part of First Circuit: INPUT
--Combinational input of Register
new_clk_counts<=clk_counts+1 when clk_counts<100000000 else 0;
--new_clk_counts<=clk_counts+1 when clk_counts<10 else 0;

--2nd Part of First Circuit: STATE (Register)
--The Register
--HOLD the clock's ticks!
counts_100M : process (clk, reset) is

variable clk_ticks : integer;

begin
    
	if reset = '1' then
	
	   clk_counts<=0;
       
	elsif rising_edge(clk) then

		clk_counts<=new_clk_counts;
		
	end if; --reset
    
end process counts_100M;

--3rd Part of First  Circuit: OUTPUT
--Compinational OUTPUT

clk_one_sec<='1' when clk_counts=100000000 else '0';
--clk_one_sec<='1' when clk_counts=10 else '0';

--SECOND LOGIC CIRCUIT
--COUNTS UP/DOWN seconds

--1st Part of Second Circuit: INPUT

process(clk_one_sec) is

begin

if (clk_one_sec='1') then 
--    report("1st sec_counts"&integer'image(sec_counts));
    if (direction='1' and sec_counts<9) then 
        new_sec_counts<= sec_counts+1; 
    elsif  (direction='0' and sec_counts>0) then
        new_sec_counts<= sec_counts-1;
    elsif  (direction='0' and sec_counts<=0) then
        new_sec_counts<= "1001"; 
    elsif (direction='1' and sec_counts>=9) then
        new_sec_counts<="0000"; 
    else
        new_sec_counts<="0000";	
    end if;
--    report("2nd sec_counts"&integer'image(sec_counts));   
else
    new_sec_counts<=sec_counts;
end if;

end process;

--2nd Part of Second Circuit: STATE (Register)
--The Register
--HOLD the seconds!
                 
count_seconds: process (clk, reset) is

begin

if (reset='1') then

    sec_counts<=(others=>'0');
--sec_counts<=0;
elsif rising_edge(clk) then

    sec_counts<=new_sec_counts;

end if; --reset

end process count_seconds;

--3rd Part of Second  Circuit: OUTPUT
--Compinational OUTPUT

---- *************************
---- COMBINATIONAL FOR OUTPUT FOR WHOLE CIRCUIT
---- *************************

--LED Output
led_result<=std_logic_vector(sec_counts);      

----PMOD Output


-- create right Pmod digit
counts_digit_values: process (sec_counts) begin

case sec_counts is

    when X"0" => seven_segment   <="0111111";    -- 0
    when X"1" => seven_segment   <="0000110";    -- 1
    when X"2" => seven_segment   <="1011011";    -- 2
    when X"3" => seven_segment   <="1001111";    -- 3
    when X"4" => seven_segment   <="1100110";    -- 4
    when X"5" => seven_segment   <="1101101";    -- 5
    when X"6" => seven_segment   <="1111101";    -- 6
    when X"7" => seven_segment   <="0000111";    -- 7
    when X"8" => seven_segment   <="1111111";    -- 8
    when X"9" => seven_segment   <="1101111";    -- 9
    when others => seven_segment <="1000000";
    
    
end case;

end process counts_digit_values;

digit_selection_out<=digit_selection_in;

end Behavioral;

